#!/usr/local/bin/python
# -*- coding: utf-8 -*-
###################################################################################################
###################################################################################################
##  Author :
##     CGTD-ICARE/UDEV Nicolas HENRIOT

##  License :
##     This file must be used under the terms of the CeCILL.
##     This source file is licensed as described in the file COPYING, which
##     you should have received as part of this distribution.  The terms
##     are also available at   http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
###################################################################################################
###################################################################################################
###################################################################################################
import numpy
import os
from datetime import date, timedelta, datetime

from dev_py.ConfigFile import *
from dev_py.ReadRawFile import *

list_of_sds_TB = ['BT_19V','BT_19H','BT_22V','BT_37V','BT_37H','BT_85V','BT_85H']
list_of_sds_SSMI = ['Latitude_BF','Longitude_BF'] + ['BT_19V','BT_19H','BT_22V','BT_37V','BT_37H'] + ['Latitude_HF','Longitude_HF']+ ['BT_85V','BT_85H']
nb_pix_per_scan = 64



from pyhdf import SD
import warnings
warnings.filterwarnings("ignore")
#-------------------------------------------------------------------------------------------------
class Writer:

    def __init__(self,data,sds_name,outfile,global_attribute):
        self.data     = data
        self.sds_name = sds_name
        self.outfile  = outfile
        self.global_attribute  = global_attribute


        # add the new dataset to out file
        self.write_sds_data(self.outfile,sds_name,data,global_attribute)


    def write_sds_data(self,filename,sds_name,data,global_attribute):
        """
        add a new dataset to the file "filename" named "sds_name" having the given data
        """
        # open the hdf file for writing
        sd = SD.SD(filename,SD.SDC.WRITE|SD.SDC.CREATE)
        hdf_type_code = self.numpy_2_hdftypecode(data.dtype)
        dataset = sd.create(sds_name,hdf_type_code,data.shape)

        #set dataset compression
        deflate_level=5
        SD.SDS.setcompress(dataset,SD.SDC.COMP_DEFLATE,deflate_level)

        #set dataset
        if data.ndim == 2:
            dataset[:,:] = data[:,:]
            #dim1 = dataset.dim(0)
            #dim2 = dataset.dim(1)
            #dim1.setname('nscan')
            #dim2.setname('npix')


        #set global attribute
        for key in global_attribute:
            setattr(sd,key,global_attribute[key])

        # Close dataset
        dataset.endaccess()

        # Close file
        sd.end()


    def numpy_2_hdftypecode(self,numpy_type_code):
        if numpy_type_code == numpy.float64:
            return SD.SDC.FLOAT64
        elif numpy_type_code == numpy.uint8:
            return SD.SDC.UINT8
        elif numpy_type_code == numpy.int16:
            return SD.SDC.INT16
        elif numpy_type_code == numpy.int32:
            return SD.SDC.INT32
        elif numpy_type_code == numpy.float32:
            return SD.SDC.FLOAT32

#-------------------------------------------------------------------------------------------------
def write_hdf_file(SSMI_L1,hdf_outfile):

    global_attribute={}

    for key in list_of_sds_SSMI:
       Writer(SSMI_L1[key],key,hdf_outfile,global_attribute)

    return

#-------------------------------------------------------------------------------------------------   
def get_L1_SSMI(cfg):


    cfg_SSMI={}
    filein = cfg['input_file']

    cfg_SSMI['HEADERS'] = os.path.join(cfg['FW_dir'],'conf/ssmi_header.txt')
    cfg_SSMI['RECORD']  = os.path.join(cfg['FW_dir'],'conf/ssmi_record.txt')
    cfg_SSMI['FILE']    = filein
    cfg_SSMI['BIGEND']  = 1

    L1={}

    len_filein=len(filein)
    a = len_filein-29
    b = len_filein-28
    c = len_filein-27
    d = len_filein-26
    e = len_filein-25
    startdatayr = 2000 + int(filein[a] + filein[b])
    startdatady = int(filein[c] + filein[d] + filein[e])
    year_start_orbite = datetime(startdatayr,1,1,0,0,0)
    date_start_orbite = year_start_orbite + timedelta(days=startdatady-1)

    headers, hdr_dscs, records, rec_dsc, scantime = ReadRawFile(cfg_SSMI)

    nb_scan = len(records)/nb_pix_per_scan
    

    year   = numpy.zeros(nb_scan,dtype=numpy.int32)
    month  = numpy.zeros(nb_scan,dtype=numpy.int32)
    day    = numpy.zeros(nb_scan,dtype=numpy.int32)
    hour   = numpy.zeros(nb_scan,dtype=numpy.int32)
    minute = numpy.zeros(nb_scan,dtype=numpy.int32)
    second = numpy.zeros(nb_scan,dtype=numpy.int32)

    n_day = 0

    for n_scan in range(nb_scan):
        if n_scan > 2 :
            if ((scantime[n_scan - 1] - scantime[n_scan]) > (23 * 60 * 60 + 59 * 60)) :
                n_day = 1

        scan_date = datetime(date_start_orbite.year, \
                             date_start_orbite.month, \
                             date_start_orbite.day, \
                             0,0,0) + timedelta(seconds=scantime[n_scan]) + timedelta(days=n_day)

        year[n_scan]   = scan_date.year
        month[n_scan]  = scan_date.month
        day[n_scan]    = scan_date.day
        hour[n_scan]   = scan_date.hour
        minute[n_scan] = scan_date.minute
        second[n_scan] = scan_date.second
        
    L1['year']   = year.repeat(2)
    L1['month']  = month.repeat(2)
    L1['day']    = day.repeat(2)
    L1['hour']   = hour.repeat(2)
    L1['minute'] = minute.repeat(2)
    L1['second'] = second.repeat(2)

    
    for sds in list_of_sds_SSMI:
        if sds in ['Latitude_HF','Longitude_HF','BT_85V','BT_85H']:
	    if sds == 'Latitude_HF'   : pos = [1,18,12,24]
            if sds == 'Longitude_HF'  : pos = [2,19,13,25]
            if sds == 'BT_85V'        : pos = [8,20,14,26]
            if sds == 'BT_85H'        : pos = [9,21,15,27]

            L1[sds]=[]
            for n_scan in range(nb_scan):
                buff_A=[]
                for n_pix in range(nb_pix_per_scan):
                    buff_A.append(records[n_scan * nb_pix_per_scan + n_pix][pos[0]])
                    buff_A.append(records[n_scan * nb_pix_per_scan + n_pix][pos[1]])
                L1[sds].append(buff_A)
                buff_B=[]
                for n_pix in range(nb_pix_per_scan):
                    buff_B.append(records[n_scan * nb_pix_per_scan + n_pix][pos[2]])
                    buff_B.append(records[n_scan * nb_pix_per_scan + n_pix][pos[3]])
                L1[sds].append(buff_B)


        if sds in ['Latitude_BF','Longitude_BF','BT_19V','BT_19H','BT_22V','BT_37V','BT_37H']:
            if sds == 'Latitude_BF'  : pos = 1
            if sds == 'Longitude_BF' : pos = 2
            if sds == 'BT_19V'       : pos = 3
            if sds == 'BT_19H'       : pos = 4
            if sds == 'BT_22V'       : pos = 5
            if sds == 'BT_37V'       : pos = 6
            if sds == 'BT_37H'       : pos = 7

            L1[sds]=[]
            for n_scan in range(nb_scan):
                buff_A=[]
                for n_pix in range(nb_pix_per_scan):
                    buff_A.append(records[n_scan * nb_pix_per_scan + n_pix][pos])
                    #buff_A.append(0.)
                L1[sds].append(buff_A)
                #buff_B=[]
                #for n_pix in range(nb_pix_per_scan):
                #    buff_B.append(0.)
                #    buff_B.append(0.)
                #L1[sds].append(buff_B)
 
    lat = numpy.array(L1['Latitude_BF'],dtype=float32)/100. - 90.
    L1['Latitude_BF'] = lat.copy()

    lon = numpy.array(L1['Longitude_BF'],dtype=float32) / 100.
    lon[ lon  >  180. ]  = lon[lon  > 180. ] - 360.
    L1['Longitude_BF'] = lon.copy()

    lat = numpy.array(L1['Latitude_HF'],dtype=float32)/100. - 90.
    L1['Latitude_HF'] = lat.copy()

    lon = numpy.array(L1['Longitude_HF'],dtype=float32) / 100.
    lon[ lon  >  180. ]  = lon[lon  > 180. ] - 360.
    L1['Longitude_HF'] = lon.copy()

    for sds in list_of_sds_TB:
        L1[sds] = numpy.array(L1[sds],dtype=float32)/100. 

    return L1
#-------------------------------------------------------------------------------------------------   
def VerifCfg(cfg):
    ok, msg = (1, "")
    if (len(cfg['FW_dir']) == 0 or
        len(cfg['input_file']) == 0 or
        len(cfg['output_dir']) == 0 ) :
        ok = 0
        msg += 'no value parameter \n'
    return (ok, msg)

#-------------------------------------------------------------------------------------------------   
#-------------------------------------------------------------------------------------------------   
def main():

    # Parameters definition
    #    name       type  option default
    req_cfg = {
       'FW_dir'                      : (  str   , 'f:', None),
       'input_file'                  : (  str   , 'i:', None),
       'output_dir'                  : (  str   , 'o:', None),
        }

    # Settings
    try:
        config = Config(VerifCfg, [""])
        cfg = config.get(req_cfg)
        
    except ConfigError, e:
        print "Configuration error; aborting..."
        return

    hdf_outfile = os.path.basename(cfg['input_file'])[:-3]+'.hdf'

    L1_SSMI = get_L1_SSMI(cfg)
    write_hdf_file(L1_SSMI,os.path.join(cfg['output_dir'],hdf_outfile))
    
if __name__ == '__main__':
    main()



